#!/usr/bin/env python
import requests
import datetime
import time
from dateutil.parser import parse
from pymusement.park import Park
from pymusement.ride import Ride
from pymusement.show import Show

PARK_URL_FORMAT = 'https://seas.te2.biz/v1/rest/venue/{0}/poi/all/status'
SHOW_URL_FORMAT = 'https://seas.te2.biz/v1/rest/venue/{0}/shows/{1}'

class SeaworldPark(Park):
    def getId(self):
        raise('Must be implemented in a subclass')

    def __init__(self):
        super(SeaworldPark, self).__init__()
        self._park_url = PARK_URL_FORMAT.format(self.getId())

        ts = time.time()
        today = datetime.datetime.fromtimestamp(ts).strftime('%Y-%m-%d')
        self._show_url = SHOW_URL_FORMAT.format(self.getId(), today)

    def _buildPark(self):
        parsed_page = self._get_page(self._park_url)
        for poi in parsed_page:
            self._build_ride(poi)

        parsed_page = self._get_page(self._show_url)
        for poi in parsed_page:
            self._build_show(poi)

    def _get_page(self, url):
        # Make page request, return Beautiful Soup request
        response = requests.get(url, auth=('seaworld', '1393288508')) 
        return response.json()

    def _build_ride(self, row):
        # Create dictionary with attraction information
        result = Ride()
        if 'label' in row:
            result.setName(row['label'])
        else:
            return

        if row['status']['isOpen']:
            result.setOpen()
        else:
            result.setClosed()
        
        if 'waitTime' in row['status']:
            result.setTime(row['status']['waitTime'])
        elif result.isOpen():
            result.setTime(0)

        self.addRide(result)

    def _build_show(self, row):
        result = Show()

        if 'title' in row:
            result.setName(row['title'])
        else:
            return

        for time_dict in row['schedule']['entries']:
            format_string = 'YYYY-MM-DDTHH:MM:SS.mmmmmm'
            time_obj = parse(time_dict['start'])
            result.addTime(time_obj)

        self.addShow(result)



#!/usr/bin/env python
from pymusement.parks.seaworld.SeaworldPark import SeaworldPark

class BuschGardensWilliamsburg(SeaworldPark):
    def __init__(self):
        super(BuschGardensWilliamsburg, self).__init__()

    def getId(self):
        return 'BG_PHF'

    def getName(self):
        return 'Busch Gardens Williamsburg'



import requests
import dateutil
from bs4 import BeautifulSoup

from pymusement.park import Park
from pymusement.show import Show
from pymusement.ride import Ride

"""
This is expirimental! It is a work in progress and probably will not work. USS's API has been very tempermental.
"""
class UniversalSingapore(Park):
    _url = 'http://cma.rwsentosa.com/Service.svc/GetUSSContent?languageID=1&filter=Show,Ride,MeetAndGreet,&Latitude=1.254251&Longitude=103.823797'

    _headers = {
        'Proxy-Connection' : 'keep-alive',
        'Accept-Encoding' : 'gzip',
        'Accept' : '*/*',
        'Accept-Language' : 'en-us',
        'Connection' : 'keep-alive',
        'User-Agent' : 'RWS/1.11 CFNetwork/758.1.6 Darwin/15.0.0'
    }
    def __init__(self):
        super(UniversalSingapore, self).__init__()

    def getName(self):
        return 'Universal Studios Singapore'

    def _buildPark(self):
        page = self.get_page(self._url)

        zone_list = page.html.body.responseofuss.result.find_all('usszonelist')[0]
        for zone in zone_list.find_all('usszone'):
            for attraction in zone.content.find_all('usscontent'):
                self._build_attr(attraction)

    def _build_attr(self, attraction):
        print attraction.contenttype.text
        if attraction.contenttype.text == 'USSShow':
            document = Show()
            document.setName(attraction.find('name').text)
            if 'ashx' in attraction.showtime.text:
                return

            showtimes = self.get_showtimes(attraction.showtime.text)
            for show in showtimes:
                try:
                    time_obj = dateutil.parser.parse(show)
                    document.addTime(time_obj)
                except:
                    pass
            self.addShow(document)

        if attraction.contenttype.text == 'Ride':
            document = Ride()
            document.setName(attraction.find('name').text)
            document.setRide()
            if 'Guests' in attraction.queuetime.text:
                document.setTime(0)
            else:
                document.setTime(attraction.queuetime.text)

            if attraction.availability.text == 'True':
                document.setOpen()
            else:
                document.setClosed()

            self.addRide(document)
            
    def get_showtimes(self, showtimes):
        showtimes = showtimes.replace('and', '')
        showtimes = showtimes.replace(',', '')
        showtimes = showtimes.split(' ')
        array_times = []
        for x in showtimes:
            time_obj = None
            if x != '':
                if 'pm' in x:
                    time_obj = ' pm'.join(x.split('pm'))
                if 'am' in x:
                    time_obj = ' am'.join(x.split('am'))

            if time_obj:
                array_times.append(time_obj)
           
        return array_times

    def get_page(self, url):
        r = requests.get(url, headers=self._headers)
        return BeautifulSoup(r.text)



#!/usr/bin/env python
from pymusement.parks.seaworld.SeaworldPark import SeaworldPark

class SeaworldOrlando(SeaworldPark):
    def __init__(self):
        super(SeaworldOrlando, self).__init__()

    def getId(self):
        return 'SW_MCO'

    def getName(self):
        return 'Seaworld Orlando'



#!/usr/bin/env python
from pymusement.parks.disney.DisneyPark import DisneyPark
class MagicKingdom(DisneyPark):

    def __init__(self):
        super(MagicKingdom, self).__init__()

    def getId(self):
        return '80007944'

    def getName(self):
        return "Disney's Magic Kingdom"



from pymusement.attraction import Attraction
class Ride(Attraction):

    _added_keys = ['wait', 'isOpen', 'single_rider', 'virtual_line']

    def __init__(self):
        super(Ride, self).__init__()
        self._addKeys(self._added_keys)

    def setOpen(self):
        self['isOpen'] = True

    def setClosed(self):
        self['isOpen'] = False

    def setTime(self, time):
        if(isinstance(time, str)):
            time = int(time)
        self['wait'] = time

    def setSingleRider(self, time):
        if(isinstance(time, str)):
            time = int(time)
        self['single_rider'] = time
    
    def has_virtual_line(self):
        if self['VirtualLine'] == True:
            return True
        else:
            return False
        
    def setVirtualLine(self, status):
        self['virtual_line'] = status
    
    def isOpen(self):
        if self['isOpen'] == True:
            return True
        else:
            return False



# -*- coding: UTF-8 -*-
strings = {
    u"フライング・スヌーピー" : u"The Flying Snoopy",
    u"スヌーピーのグレートレース™" : u"Snoopy's Great Race",
    u"ハローキティのカップケーキ・ドリーム" : u"Hello Kitty's Cupcake Dream",
    u"ジュラシック・パーク・ザ・ライド®\n※ご利用には整理券が必要です" : u"Jurassic Park: The Ride",
    u"バック・トゥ・ザ・フューチャー®・ザ・ライド" : u"Back to the Future: The Ride",
    u"フライト・オブ・ザ・ヒッポグリフ™" : u"Flight of the Hippogriff",
    u"スペース・ファンタジー・ザ・ライド" : u"Space Fantasy: The Ride",
    u"アメージング・アドベンチャー・オブ・スパイダーマン・ザ・ライド 4K3D" : u"The Amazing Adventures of Spiderman: 4K3D",
    u"ハリウッド・ドリーム・ザ・ライド" : u"Hollywood Dream: The Ride",
    u"ハリー・ポッター・アンド・ザ・フォービドゥン・ジャーニー™" : u"Harry Potter and the Forbidden Journey",
    u"ハリウッド・ドリーム・ザ・ライド～バックドロップ～" : u"Hollywood Dream: The Ride - Backdrop",
    u"バックドラフト®" : u"Backdraft",
    u"ジョーズ®" : u"Jaws",
    u"エヴァンゲリオン・ザ・リアル 4-D": u"Evangelion-the-Real 4-D",
    u"セサミストリート 4-D ムービーマジック™": u"Sesame Street 4-D Movie Magic",
    u"フロッグ・クワイア": u"Frog Choir",
    u"ウォーターワールド®": u"Waterworld",
    u"バイオハザード®・ザ・エスケープ\n※専用チケット（有料）が必要です" : u"Resident Evil",
    u"リアル・アイルー" : u"Real Airou",
    u"トライウィザード・スピリット・ラリー" : u"Tri-Wizard Spirit Rally",
    u"ユニバーサル・モンスター・ライブ・ロックンロール・ショー®" : u"Universal Monster Live Rock and Roll Show",
    u"日本アニメ（ーター）見本市\nIN UNIVERSAL STUDIOS JAPAN" : u"Japanese Animation Fair",
    u"休止中" : u"Closed",
    u"ターミネーター 2:3-D®" : u"Terminator 2 3:D",
    u"モッピーのバルーン・トリップ" : u"Moppy's Balloon Trip",
    u"エルモのゴーゴー・スケートボード" : u"Elmo's Go-Go Skateboard",
    u"R&B\"ザ・ミックス\"" : u"R&B The Mix",
    u"フュージョン・ダンス・ビート" : u"Fusion Dance Beat",
    u"ミニオン・フィーバー" : u"Minion Fever",
    u"クッキーモンスターのくいしんぼうクッキング\n※専用のチケット(有料)が必要です" : u"Cookie Monster Slide",
    u"フラッシュ・バンド・ビート 2015" : u"Flash Band Beat 2015",
    u"マジカル・スターライト・パレード" : u"Magical Starlight Parade",
    u"ザ・ヴァイオリン・ジャム・カルテット" : u"The Violin Jam Quartet"

}



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import requests
import zipfile
from io import StringIO
import json
from pymusement.park import Park
from pymusement.ride import Ride
from pymusement.config.disneyparis import strings

class DisneylandParis(Park):
    def __init__(self):
        super(DisneylandParis, self).__init__()

    def getName(self):
        return 'Disneyland Paris'

    def _buildPark(self):
        times = self._get_page()
        time_list = times['l']
        for i in range(0, len(times['l']), 5):
            self._make_attr(time_list[i:i+5])

    def _get_page(self):
        body = 'key=Ajjjsh;Uj'

        headers = {
            'Host' : 'disney.cms.pureagency.com',
            'Proxy-Connection': 'close',
            'Accept-Encoding' : 'gzip',
            'Content-Type': 'application/x-www-form-urlencoded;charset=UTF-8',
            'Content-Length' : 18,
            'Connection' : 'close',
            'User-Agent': 'Disneyland 1.0 (iPhone; iPhone OS 4.1; en_GB'
        }

        r = requests.post('https://disney.cms.pureagency.com/cms/ProxyTempsAttente', headers=headers, data=body)
        z = zipfile.ZipFile(StringIO.StringIO(r.content)).open('temps_attente.json')
        return json.loads(z.read())

    def _make_attr(self, array):
        if array[0] not in strings:
            return

        else:
            attr_doc = Ride()
            attr_doc.setName(strings[array[0]])
            if array[3] == 1:
                attr_doc.setOpen()
            else:
                attr_doc.setClosed()

            attr_doc.setTime(array[4])
            self.addRide(attr_doc)



#!/usr/bin/env python
import requests
from pymusement.park import Park
from pymusement.ride import Ride

class HersheyPark(Park):
    _park_url = 'https://hpapp.hersheypa.com/v1/rides'
    _wait_url = 'https://hpapp.hersheypa.com/v1/rides/wait' 


    def __init__(self):
        super(HersheyPark, self).__init__()

    def getName(self):
        return 'Hersheypark'

    def _buildPark(self):
        park_page = self.get_page(self._park_url)
        wait_page = self.get_page(self._wait_url)
        # Deal with attractions
        for poi in park_page:
            self._build_attr(poi, wait_page)


    def get_page(self, url):
        # Make page request, return Beautiful Soup request
        response = requests.get(url, timeout=3)
        return response.json()

    def _build_attr(self, row, wait_page):
        # Create dictionary with attraction information
        result = Ride()

        result.setName(row['name'])        
        result.setOpen()
        result.setTime(0)

        if 'wait' in wait_page:
            for wait_dict in wait_page['wait']:
                if wait_dict['id'] == row['id']:
                    result.setTime(wait_dict['wait'])

        if 'closed' in wait_page:
            for wait in wait_page['closed']:
                if wait == row['id']:
                    result.setClosed()

        if 'wait' in wait_page and len(wait_page['wait']) == 0:
            result.setClosed()

        self.addRide(result)



#!/usr/bin/env python
from pymusement.parks.disney.DisneyPark import DisneyPark
class AnimalKingdom(DisneyPark):

    def __init__(self):
        super(AnimalKingdom, self).__init__()

    def getId(self):
        return '80007823'

    def getName(self):
        return "Disney's Animal Kingdom"



#!/usr/bin/env python
from pymusement.parks.seaworld.SeaworldPark import SeaworldPark

class SeaworldSanAntonio(SeaworldPark):
    def __init__(self):
        super(SeaworldSanAntonio, self).__init__()

    def getId(self):
        return 'SW_SAT'

    def getName(self):
        return 'Seaworld San Antonio'



#!/usr/bin/env python
import requests
import json
import datetime
from pymusement.park import Park
from pymusement.ride import Ride

URL_TEMPLATE = 'https://api.wdpro.disney.go.com/facility-service/theme-parks/%s'

class DisneyPark(Park):

    def __init__(self):
        self._id = self.getId()
        self._url = URL_TEMPLATE % self._id
        self.name = self.getName()
        super(DisneyPark, self).__init__()

    def getId(self):
        raise('This must be implemented in a subclass')

    def getName(self):
        raise('This must be implemented in a subclass')

    def _buildPark(self):
        self._auth_token = self._authorize()
        page = self._get_page()
        for attraction in page['entries']:
            if attraction['type'] != 'Attraction':
                continue
            self._build_attraction(attraction)

    def _build_attraction(self, attraction):
        attraction_obj = Ride()

        attraction_obj.setName(attraction['name'])
        self._waitTime(attraction_obj, attraction)

        self.addRide(attraction_obj)
    
    
    
    def _get_page(self):
        # Make page request, return dictionary of a park's waittimes.
        
        coach_headers = header = {"Authorization":"Basic RFBFQ1AtTU9CSUxFLldEVy5BTkRST0lELVBST0Q6RGhyeHMyZHVReGdiVjZ5Mg==","User-Agent":"CouchbaseLite/1.3 (1.4.1/8a21c5927a273a038fb3b66ec29c86425e871b11)","Content-Type":"application/json","Accept":"multipart/related"}
        headers = {
            'Accept-Language' : 'en_US',
            'User-Agent': 'UIEPlayer/2.1 iPhone OS 6.0.1',
            'Accept' : 'application/json;apiversion=1',
            'Connection' : 'keep-alive',
            'Proxy-Connection' : 'keep-alive',
            'Accept-Encoding' :'compress, gzip',
            'Authorization' :'BEARER ' + self._authorize(),
            'X-Conversation-Id' : 'WDPRO-MOBILE.MDX.CLIENT-PROD',
            'X-Correlation-ID' : str(datetime.datetime.now().timestamp())
        }
        
        channel = 'wdw.facilities.1_0.en_us'
        
        payload = {
            "channels": channel,
            "style": 'all_docs',
            "filter": 'sync_gateway/bychannel',
            "feed": 'normal',
            "heartbeat": 30000
        }
        
        r = requests.post("https://realtime-sync-gw.wdprapps.disney.com/park-platform-pub/_changes?feed=normal&heartbeat=30000&style=all_docs&filter=sync_gateway%2Fbychannel", data=json.dumps(payload), headers = coach_headers)
        s = json.loads(r.text)
        docs = []
        for i in s['results']:
            try:
                i['deleted']
                continue
            except:
                this = {}
                this['id'] = i['id']

                docs.append(this)

                split_id = i['id'].split(":")
                if len(split_id) > 1:
                    this = {}
                    this['id'] = split_id[0]
                    docs.append(this)
        payload = {"docs": docs, "json":True}
        requests.post("https://realtime-sync-gw.wdprapps.disney.com/park-platform-pub/_bulk_get?revs=true&attachments=true", data=json.dumps(payload), headers=coach_headers())
        s = r.text

        cont_reg = re.compile("\w+-\w+:\s\w+\/\w+")
        s = re.sub(cont_reg, "", s)
        s = s.splitlines()
        for x in s:
            if x != '' and x[0] != '-':
                try:
                    x = json.loads(x)
                    c.execute("INSERT INTO sync (id, rev, body, channel) VALUES (?, ?, ?, ?)", (x['_id'], x['_rev'], json.dumps(x), channel,))

                    split_id = x['_id'].split(':')
                    this['id'] = split_id[-1].split(';')[0].split('.')[-1]


                    this['name'] = x['name']
                    this['entityType'] = x['type']

                    try:
                        this['sub'] = x['subType']
                    except:
                        this['sub'] = None

                    this['cb_id'] = x['_id']
                    this['dest_code'] = x['_id'].split('.')[0]

                    try:
                        this['park_id'] = x['ancestorThemeParkId'].split(';')[0]
                    except:
                        try:
                            this['park_id'] = x['ancestorWaterParkId'].split(';')[0]
                        except:
                            this['park_id'] = None

                    try:
                        this['land_id'] = x['ancestorLandId'].split(';')[0]
                    except:
                        this['land_id'] = None

                    try:
                        this['resort_id'] = x['ancestorResortId'].split(';')[0]
                    except:
                        this['resort_id'] = None

                    try:
                        this['ra_id'] = x['ancestorResortAreaId'].split(';')[0]
                    except:
                        this['ra_id'] = None

                    try:
                        this['ev_id'] = x['ancestorEntertainmentVenueId'].split(';')[0]
                    except:
                        this['ev_id'] = None
                except Exception as e:
                    # print(x)
                    # print(e)
                    continue
        ride_link = self._url + '/wait-times'
        page = requests.get(self._url, headers=headers)
        response = requests.get(ride_link, headers=headers)
        print(response)
        
        return page

    def _waitTime(self,attraction_doc, attraction):
        if 'waitTime' in attraction:
            attraction_doc.setOpen()
            attraction_doc.setTime(0)
            if 'status' in attraction['waitTime'] and attraction['waitTime']['status'] == 'Closed':
                attraction_doc.setClosed()
            if 'actualWaitMinutes' in attraction['waitTime']:
                attraction_doc.setTime(attraction['waitTime']['actualWaitMinutes'])
            elif 'postedWaitMinutes' in attraction['waitTime']:
                attraction_doc.setTime(attraction['waitTime']['postedWaitMinutes'])
        else:
            attraction_doc.setClosed()
        

    def _fastPass(self,attraction_doc, attraction):
        if 'waitTime' in attraction:
            if 'fastpass' in attraction['waitTime']:
                attraction_doc.setFastPass(attraction['waitTime']['fastpass'])

    def _singleRider(self,attraction_doc, attraction):
        if 'singleRider' in attraction:
            attraction_doc.setSingleRider(attraction['singleRider'])
        

    def _authorize(self):
        # Returns authorization token

        r = requests.get("https://disneyworld.disney.go.com/authentication/get-client-token")
        response = json.loads(r.content)
        auth_token = response['access_token']
        return auth_token
